{
  "name-project":"",
  "version-c++":"11",
  "version-qt":"5.10.0",
  "version-cmake":"3.1.0",
  "libraries-qt":["Core"],
  "version-major-app":"1",
  "version-minor-app":"0",
  "version-patch-app":"0",
  "organization-app":"organization",
  "domain-app":"domain",
  "name-icon-executable":"icon.icns"
}